
BeginPackage["ParticleTrack`"]

EParticleTrack::usage = 
"EParticleTrack[{v1,v2,..}, {x1,x2,..}, {a1,a2,..}, {t1, dt}]\n
numerically integrates the vi as functions of the \n
coordinates xi with initial values ai. The integration \n
proceeds in steps of dt from 0 to t1. The vi are to be \n
thought of as holonomic gradients of the particle \n
potential. The simple Euler method is used.\n\n
EParticleTrack[{v1,v2,..}, {x1,x2,..}, {a1,a2,..},\n
\t\t{t, t0, t1, dt}]\n
performs the calculation over time t proceeding in steps \n
of dt from t0 to t1. A negative dt yields particle tracking \n
backwards in time.\n\n
EParticleTrack[{v1,v2,..}, {x1,x2,..}, {a1,a2,..}, \n
\t\t{t1, dt}, {c1,c2,..}, tol, \n
\t\t{{x1min, x1max},{x2min,x2max},..}]\n
performs the particle tracking to within an accuracy tol of \n
the specified critical point(s) with coordinates ci. The \n
domain is bounded by the min and max arguments.\n\n
EParticleTrack[{v1,v2,..}, {x1,x2,..}, {a1,a2,..},\n
\t\t{t, t0, t1, dt}, {c1,c2,..}, tol, \n
\t\t{{x1min, x1max},{x2min,x2max},..}]\n
performs the particle tracking to within tolerance tol of \n
the specified critical point(s) with coordinates ci in \n
steps of dt from t0 to t1. The domain is bounded by the \n
min and max arguments. The critical point coordinates ci\n
may be given in vector or matrix form. That is, an \n
arbitrary number of critical points may be given."

RKParticleTrack::usage = 
"RKParticleTrack[{v1,v2,..}, {x1,x2,..}, {a1,a2,..},\n
\t\t{t1, dt}]\n
numerically integrates the vi as functions of the \n
coordinates xi with initial values ai. The integration \n
proceeds in steps of dt from 0 to t1. The vi are to be \n
thought of as holonomic gradients of the particle \n
potential. The fourth order Runge-Kutta method is used.\n\n
RKParticleTrack[{v1,v2,..}, {x1,x2,..}, {a1,a2,..},\n
\t\t{t, t0, t1, dt}]\n
performs the calculation over time t proceeding in steps \n
of dt from t0 to t1. A negative dt yields particle tracking \n
backwards in time.\n\n
RKParticleTrack[{v1,v2,..}, {x1,x2,..}, {a1,a2,..}, \n
\t\t{t1, dt}, {c1,c2,..}, tol, \n
\t\t{{x1min, x1max},{x2min,x2max},..}]\n
performs the particle tracking to within an accuracy tol of \n
the specified critical point(s) with coordinates ci. The \n
domain is bounded by the min and max arguments.\n\n
RKParticleTrack[{v1,v2,..}, {x1,x2,..}, {a1,a2,..},\n
\t\t{t, t0, t1, dt}, {c1,c2,..}, tol, \n
\t\t{{x1min, x1max},{x2min,x2max},..}]\n
performs the particle tracking to within tolerance tol of \n
the specified critical point(s) with coordinates ci in \n
steps of dt from t0 to t1. The domain is bounded by the \n
min and max arguments. The critical point coordinates ci\n
may be given in vector or matrix form. That is, an \n
arbitrary number of critical points may be given."

Criticals::usage = 
"Criticals[p_List, c_list, tolerance, b_list] \n
truncates the supplied list of points p within\n
distance tolerance of any of the supplied critical\n
points c. The list p is stepped through sequentially.\n
The bounds b are given as a list of {min,max} pairs \n
for each coordinate."

Begin["`Private`"]

EPTStep[f_, x_, x0_, dt_] :=
	Module[{ dx },
		dx = dt N[ f /. Thread[x -> x0] ];
		x0 + Chop[dx]
	]

RKStep[f_, x_, x0_, dt_] :=
	Module[{ k1, k2, k3, k4 },
		k1 = dt N[ f /. Thread[x -> x0] ];
		k2 = dt N[ f /. Thread[x -> x0 + Chop[k1/2]] ];
		k3 = dt N[ f /. Thread[x -> x0 + Chop[k2/2]] ];
		k4 = dt N[ f /. Thread[x -> x0 + Chop[k3]] ];
		x0 + Chop[(k1 + 2 k2 + 2 k3 + k4)/6]
	]

EParticleTrack[f_List, x_List, x0_List, {t1_, dt_}] :=
	NestList[ EPTStep[f, x, #, N[dt]]&, N[x0], Round[N[t1/dt]] ] /;
		Length[f] == Length[x] == Length[x0] /;
		Sign[t1] == Sign[dt]

RKParticleTrack[f_List, x_List, x0_List, {t1_, dt_}] :=
	NestList[ RKStep[f, x, #, N[dt]]&, N[x0], Round[N[t1/dt]] ] /;
		Length[f] == Length[x] == Length[x0] /;
		Sign[t1] == Sign[dt]

EParticleTrack[f_List, x_List, x0_List, {t_, t0_, t1_, dt_}] :=
	Module[{ res },
		res = EParticleTrack[ Append[f, 1], Append[x, t], Append[x0, t0], {t1 - t0, dt} ];
		Drop[#, -1]& /@ res
	]  /;  Length[f] == Length[x] == Length[x0]

RKParticleTrack[f_List, x_List, x0_List, {t_, t0_, t1_, dt_}] :=
	Module[{ res },
		res = RKParticleTrack[ Append[f, 1], Append[x, t], Append[x0, t0], {t1 - t0, dt} ];
		Drop[#, -1]& /@ res
	]  /;  Length[f] == Length[x] == Length[x0]

Criticals[p_List, c_List, tolerance_,b_List] :=
	Which[ VectorQ[c], Module[ {plast, critpt,i,j,k},
				   critpt - False;
		                   Do[ plast = i;
		                       If[ Sqrt[ (c - p[[i]]) . (c - p[[i]]) ] < tolerance,
		                           critpt = True; Break[]
		                       ];
				       Do[ If[ p[[i,k]] <= b[[k,1]] || p[[i,k]] >= b[[k,2]],
					       critpt = True; plast = i - 1; Break[]
				           ],
					   {k, 1, Length[p[[1]]]}
				       ];
		                       If[ critpt, Break[] ],
		                       {i, 1, Length[p]}
		                   ];
		                   Drop[ p, {plast + 1, Length[p]}]
	                   ],
	       MatrixQ[c], Module[ {plast, critpt,i,j,k},
		                   critpt = False;
		                   Do[ plast = i;
				       Do[ If[ p[[i,k]] <= b[[k,1]] || p[[i,k]] >= b[[k,2]],
				               critpt = True; plast = i - 1; Break[]
				           ],
				       {k, 1, Length[p[[1]]]}
				       ];
		                       Do[ If[ Sqrt[ (c[[j]] - p[[i]]) . (c[[j]] - p[[i]]) ] < tolerance,
			                       critpt = True; Break[]
				           ],
			                   {j, 1, Length[c]}
		                       ];
		                       If[ critpt, Break[] ],
		                       {i, 1, Length[p]}
		                   ];
		                   Drop[ p, {plast + 1, Length[p]}]
	                   ]
	];

EParticleTrack[f_List, x_List, x0_List, {t1_, dt_}, c_List, tolerance_, b_List] :=
	Criticals[ EParticleTrack[f, x, x0, {t1, dt}], c, tolerance, b]

EParticleTrack[f_List, x_List, x0_List, {t_, t0_, t1_, dt_}, c_List, tolerance_, b_List] :=
	Criticals[ EParticleTrack[f, x, x0, {t, t0, t1, dt}], c, tolerance, b]

RKParticleTrack[f_List, x_List, x0_List, {t1_, dt_}, c_List, tolerance_, b_List] :=
	Criticals[ RKParticleTrack[f, x, x0, {t1, dt}], c, tolerance, b]

RKParticleTrack[f_List, x_List, x0_List, {t_, t0_, t1_, dt_}, c_List, tolerance_, b_List] :=
	Criticals[ RKParticleTrack[f, x, x0, {t, t0, t1, dt}], c, tolerance, b]

End[]

Protect[EParticleTrack,RKParticleTrack,Criticals]

EndPackage[]
